import unittest
import bruneus
from dotenv import load_dotenv

load_dotenv()


class TestSelect(unittest.TestCase):
    def test_select_with_no_params(self):
        query = "SELECT 1"
        task = bruneus.select(query)
        self.assertEqual(query, task.query)

    def test_select(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare` order by rand() limit 5"
        words = bruneus.select(query).as_dicts()
        self.assertEqual(len(words), 5)

    def test_select_with_no_result(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare` where word = 'abcdef-012345' order by rand() limit 5"
        word = bruneus.select(query).first_as_dict()
        self.assertIsNone(word)

    def test_select_with_template(self):
        words = (
            bruneus.select_with_template("./tests/templates", "shakespeare.j2")
            .int64_param("max_count", 3)
            .as_dicts()
        )
        self.assertEqual(len(words), 3)

    def test_select_to_dataframe(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare` order by rand() limit 5"
        df = bruneus.select(query).to_dataframe()
        self.assertEqual(len(df.index), 5)

    def test_select_to_stringified_array(self):
        query = """
        SELECT
        word, word_count
        FROM `bigquery-public-data.samples.shakespeare` 
        WHERE 
        LENGTH(word) >= 5 AND word_count >= 30
        ORDER BY word LIMIT 5
        """
        strings = bruneus.select(query).stringify(
            names={
                "word": "Word",
                "word_count": "Word Count",
            },
            delimiter="\n",
            format="{{ name }}({{ key }}): {{ value }}",
        )
        self.assertEqual(len(strings), 5)

        for s in strings:
            self.assertIn("Word(", s)
            self.assertIn("Word Count(", s)

    def test_select_to_stringified_array_on_empty_data(self):
        query = """
        SELECT
        word, word_count
        FROM `bigquery-public-data.samples.shakespeare` 
        WHERE 
        LENGTH(word) >= 99999
        ORDER BY word LIMIT 5
        """
        strings = bruneus.select(query).stringify(
            names={
                "word": "Word",
                "word_count": "Word Count",
            },
            delimiter="\n",
            format="{{ name }}({{ key }}): {{ value }}",
        )
        self.assertEqual(len(strings), 0)

    def test_select_int64_param(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare` order by rand() limit @max_count"
        select_task = bruneus.select(query)
        self.assertEqual(len(select_task.query_parameters), 0)
        select_task.int64_param("max_count", 5)
        self.assertEqual(len(select_task.query_parameters), 1)

    def test_select_int64_array_param(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare` WHERE word_count in UNNEST(@word_counts) order by rand() LIMIT 5"
        select_task = bruneus.select(query)
        self.assertEqual(len(select_task.query_parameters), 0)
        select_task.int64_array_param("word_counts", [10, 20, 30])
        self.assertEqual(len(select_task.query_parameters), 1)

    def test_select_str_array_param(self):
        query = "SELECT DISTINCT word FROM `bigquery-public-data.samples.shakespeare` WHERE word in UNNEST(@words)"
        select_task = bruneus.select(query)
        self.assertEqual(len(select_task.query_parameters), 0)
        select_task.str_array_param("words", ["quality", "that", "your"])
        self.assertEqual(len(select_task.query_parameters), 1)
        self.assertEqual(len(select_task.to_dataframe().index), 3)

    def test_select_float64_param(self):
        query = "SELECT count(1) FROM `bigquery-public-data.samples.gsod` WHERE mean_temp > @mean_temp"
        select_task = bruneus.select(query)
        self.assertEqual(len(select_task.query_parameters), 0)
        select_task.float64_param("mean_temp", 50.0)
        self.assertEqual(len(select_task.query_parameters), 1)
        self.assertEqual(len(select_task.to_dataframe().index), 1)

    def test_select_params(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare` order by rand() limit @max_count"
        task1 = bruneus.select(query).int64_param("max_count", 1)
        self.assertEqual(len(task1.query_parameters), 1)

        task2 = bruneus.select(query).int64_param("max_count", 2)
        self.assertEqual(len(task2.query_parameters), 1)

    def test_select_dry_run(self):
        query = "SELECT word FROM `bigquery-public-data.samples.shakespeare`"
        result = bruneus.select(query).dry_run()
        self.assertTrue(result.total_bytes_processed > 0)

if __name__ == "__main__":
    unittest.main()



from google.cloud import bigquery
from jinja2 import Template, Environment, FileSystemLoader
from .create_table import create_table
import google.cloud.bigquery


class SelectTask:
    def __init__(self, client, query):
        self.client = client
        self.query = query
        self.query_parameters = []

    def params(self, params):
        self.query_parameters = params
        return self

    def add_scalar_param(self, key: str, value: int, data_type: str):
        if self.query_parameters is None:
            self.query_parameters = []

        self.query_parameters.append(
            bigquery.ScalarQueryParameter(key, data_type, value),
        )
        return self

    def int64_param(self, key: str, value: int):
        return self.add_scalar_param(key, value, "INT64")

    def str_param(self, key: str, value: str):
        return self.add_scalar_param(key, value, "STRING")

    def float64_param(self, key: str, value: float):
        return self.add_scalar_param(key, value, "FLOAT64")

    def add_array_param(self, key: str, values: list, data_type: str):
        if self.query_parameters is None:
            self.query_parameters = []

        self.query_parameters.append(
            bigquery.ArrayQueryParameter(key, data_type, values),
        )
        return self

    def int64_array_param(self, key: str, values: list[int]):
        return self.add_array_param(key, values, "INT64")

    def str_array_param(self, key: str, values: list[str]):
        return self.add_array_param(key, values, "STRING")

    def float64_array_param(self, key: str, values: list[float]):
        return self.add_array_param(key, values, "FLOAT64")

    def bq_client(self):
        return self.client if self.client is not None else bigquery.Client()

    def job(self, dry_run=False, use_query_cache=True):
        job_config = bigquery.QueryJobConfig(query_parameters=self.query_parameters, dry_run=dry_run, use_query_cache=use_query_cache)
        return self.bq_client().query(self.query, job_config=job_config)

    def as_dicts(self):
        query_job = self.job()
        return [dict(row) for row in query_job]

    def stringify(
        self, names={}, delimiter="\n", format="{{ name }}({{ key }}): {{ value }}"
    ):
        dicts = self.as_dicts()
        strings = []
        for d in dicts:
            formatted_fields = []
            for k, v in d.items():
                name = names.get(k, k)
                formatted_str = Template(format).render(key=k, name=name, value=v)
                formatted_fields.append(formatted_str)
            strings.append(delimiter.join(formatted_fields))
        return strings

    def first_as_dict(self):
        dicts = self.as_dicts()
        if len(dicts) == 0:
            return None
        return self.as_dicts()[0]

    def to_table(self, table_id, overwrite=False):
        return create_table(
            table_id,
            self.query,
            self.query_parameters,
            overwrite=overwrite,
            client=self.client,
        )

    def to_dataframe(self):
        return self.job().to_dataframe()

    def to_df(self):
        return self.to_dataframe()

    def dry_run(self, use_query_cache=False):
        return self.job(dry_run=True, use_query_cache=use_query_cache)


def select(
    query: str,
    query_parameters=None,
    client: google.cloud.bigquery.Client = None,
):
    return SelectTask(client=client, query=query).params(
        [] if query_parameters is None else query_parameters
    )


def select_with_template(
    template_dir: str, template_name: str, template_parameters={}, client=None
):
    env = Environment(loader=FileSystemLoader(template_dir, encoding="utf8"))
    tmpl = env.get_template(template_name)
    query = tmpl.render(template_parameters)

    return select(query=query, query_parameters=[], client=client)
